@@ -1,8 +1,8 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Callable, ClassVar, Dict, Iterator, List, Optional, Tuple, Type, cast
+from typing import Any, Callable, ClassVar, Dict, Iterator, List, Optional, Tuple, Type, cast
 
-from typing_extensions import TypeAlias, TypeGuard
+from typing_extensions import TypeGuard
 
 from chalk.features.feature_field import Feature
 from chalk.features.feature_wrapper import FeatureWrapper
@@ -109,19 +109,12 @@ class FeaturesImpl(metaclass=FeaturesMeta):
         raise NotImplementedError
 
 
-# if TYPE_CHECKING:
-#     Features = FeaturesImpl
-# else:
-#     # Hack to get intellij to not complain about Features[...] when used as a return annotation
-#     # Intellij ignores the __getitem__ on the metaclass, causing it to want all return values
-#     # to be Features instances, when we accept literal values or tuples
-#     #
-#     # Pylance is able to parse through this lambda and type Features as
-#     # Type[Features] | Type[object], which causes typing issues later because
-#     # object is not subscriptable. Hopefully IntelliJ ignroes the if TYPE_CHECKING
-#     # above
-# n.b. 12/4/22: above change seemed to regress intellij.
-Features: TypeAlias = (lambda xyz: FeaturesImpl if xyz > 1 else object)(2)
+# Hack to get VSCode/Pylance/Pyright to type Features as Type[FeatureImpl]
+# but IntelliJ to type it as Type[Any]
+# Vscode can parse through literal dicts; IntelliJ can't
+_dummy_dict = {"0": FeaturesImpl}
+
+Features = _dummy_dict["0"]
 
 
 class FeatureSetBase:
