@@ -1,5 +1,4 @@
 import ast
-import copy
 import functools
 import inspect
 import re
@@ -30,7 +29,6 @@ from chalk.features.feature_wrapper import FeatureWrapper
 from chalk.features.tag import Tags
 from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.streams import Windowed
-from chalk.streams._windows import WindowedMeta
 from chalk.utils.collections import ensure_tuple
 from chalk.utils.duration import Duration, timedelta_to_duration
 from chalk.utils.metaprogramming import MISSING, create_fn, field_assign, set_new_attribute
@@ -118,7 +116,8 @@ def features(
         )
         set_new_attribute(cls=updated_class, name="__chalk_primary__", value=primary_feature)
         set_new_attribute(cls=updated_class, name="__chalk_ts__", value=timestamp_feature)
-        FeatureSetBase.hook and FeatureSetBase.hook(cast(Features, updated_class))
+        if FeatureSetBase.hook is not None:
+            FeatureSetBase.hook(cast(Features, updated_class))
 
         return cast(Type[T], updated_class)
 
