@@ -1,7 +1,7 @@
 import dataclasses
 import inspect
 from dataclasses import dataclass
-from inspect import Parameter
+from inspect import Parameter, isclass
 from typing import (
     Any,
     Callable,
@@ -22,10 +22,11 @@ from typing import (
     overload,
 )
 
+import polars
+import pyarrow
 from pydantic import BaseModel
 from typing_extensions import ParamSpec
 
-from chalk.features import FeatureSetBase
 from chalk.features.dataframe import DataFrame
 from chalk.features.feature_field import Feature
 from chalk.features.feature_set import Features
@@ -519,6 +520,60 @@ def online(
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
 ) -> Union[Callable[[Callable[P, T]], Callable[P, T]], Callable[P, T]]:
+    """
+    :param fn: The function that you're decorating as a resolver.
+
+    :param environment: Environments are used to trigger behavior
+        in different deployments such as staging, production, and
+        local development. For example, you may wish to interact with
+        a vendor via an API call in the production environment, and
+        opt to return a constant value in a staging environment.
+
+        Environment can take one of three types:
+
+            - None (default) - candidate to run in every environment
+            - str - run only in this environment
+            - list[str] - run in any of the specified environment and no others
+
+        Read more at https://docs.chalk.ai/docs/resolver-environments
+
+    :param tags: Tags allow you to scope requests within an
+        environment. Both tags and environment need to match for a
+        resolver to be a candidate to execute.
+
+        You might consider using tags, for example, to change out
+        whether you want to use a sandbox environment for a vendor,
+        or to bypass the vendor and return constant values in a
+        staging environment.
+
+        Read more at https://docs.chalk.ai/docs/resolver-tags
+
+    :param cron: You can schedule resolvers to run on a
+        pre-determined schedule via the cron argument to resolver
+        decorators.
+
+        Cron can sample all examples, a subset of all examples,
+        or a custom provided set of examples.
+
+        Read more at https://docs.chalk.ai/docs/resolver-cron
+
+    :param machine_type: You can optionally specify that resolvers
+        need to run on a machine other than the default. Must be
+        configured in your deployment.
+
+    :param when: Like tags, `when` can filter when a resolver
+        is eligible to run. Unlike tags, `when` can use feature values,
+        so that you can write resolvers like:
+
+            @batch(when=User.risk_profile == "low" or User.is_employee)
+            def resolver_fn(...) -> ...:
+                ...
+
+    :return: A callable function! You can unit test resolvers
+        as you would unit test any other code.
+
+    Read more at https://docs.chalk.ai/docs/unit-tests
+    """
     caller_frame = inspect.stack()[1]
     caller_filename = caller_frame.filename
     caller_globals = caller_frame.frame.f_globals
@@ -550,7 +605,8 @@ def online(
         )
 
         Resolver.registry.append(resolver)
-        Resolver.hook and resolver.hook(resolver)
+        if Resolver.hook:
+            Resolver.hook(resolver)
 
         # Return the decorated resolver, which notably implements __call__() so it acts the same as
         # the underlying function if called directly, e.g. from test code
@@ -589,6 +645,60 @@ def offline(
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
 ) -> Union[Callable[[Callable[P, T]], Callable[P, T]], Callable[P, T]]:
+    """
+    :param fn: The function that you're decorating as a resolver.
+
+    :param environment: Environments are used to trigger behavior
+        in different deployments such as staging, production, and
+        local development. For example, you may wish to interact with
+        a vendor via an API call in the production environment, and
+        opt to return a constant value in a staging environment.
+
+        Environment can take one of three types:
+
+            - None (default) - candidate to run in every environment
+            - str - run only in this environment
+            - list[str] - run in any of the specified environment and no others
+
+        Read more at https://docs.chalk.ai/docs/resolver-environments
+
+    :param tags: Tags allow you to scope requests within an
+        environment. Both tags and environment need to match for a
+        resolver to be a candidate to execute.
+
+        You might consider using tags, for example, to change out
+        whether you want to use a sandbox environment for a vendor,
+        or to bypass the vendor and return constant values in a
+        staging environment.
+
+        Read more at https://docs.chalk.ai/docs/resolver-tags
+
+    :param cron: You can schedule resolvers to run on a
+        pre-determined schedule via the cron argument to resolver
+        decorators.
+
+        Cron can sample all examples, a subset of all examples,
+        or a custom provided set of examples.
+
+        Read more at https://docs.chalk.ai/docs/resolver-cron
+
+    :param machine_type: You can optionally specify that resolvers
+        need to run on a machine other than the default. Must be
+        configured in your deployment.
+
+    :param when: Like tags, `when` can filter when a resolver
+        is eligible to run. Unlike tags, `when` can use feature values,
+        so that you can write resolvers like:
+
+            @batch(when=User.risk_profile == "low" or User.is_employee)
+            def resolver_fn(...) -> ...:
+                ...
+
+    :return: A callable function! You can unit test resolvers
+        as you would unit test any other code.
+
+    Read more at https://docs.chalk.ai/docs/unit-tests
+    """
     caller_frame = inspect.stack()[1]
     caller_filename = caller_frame.filename
     caller_globals = caller_frame.frame.f_globals
@@ -618,7 +728,8 @@ def offline(
             default_args=parsed.default_args,
         )
         Resolver.registry.append(resolver)
-        Resolver.hook and resolver.hook(resolver)
+        if Resolver.hook:
+            Resolver.hook(resolver)
         return fn
 
     return decorator(fn) if fn else decorator
@@ -707,6 +818,7 @@ class StreamResolver(Resolver[P, T]):
         environment: Optional[List[str]],
         machine_type: Optional[str],
         output: Type[Features],
+        message: Optional[Type[Any]],
     ):
         parsed = parse_function(fn, caller_globals, caller_locals, allow_custom_args=True, is_streaming_resolver=True)
         self.function_definition = parsed.function_definition
@@ -726,6 +838,7 @@ class StreamResolver(Resolver[P, T]):
         self.tags = None  # TODO -- should there be tags?
         self.signature = signature
         self.output = output
+        self.message = message
         self.default_args = []
         fqn_to_windows = {o.fqn: o.window_durations for o in _flatten_features(self.output) if o.is_windowed}
         if len(set(tuple(v) for v in fqn_to_windows.values())) > 1:
@@ -773,13 +886,15 @@ def _parse_stream_resolver_param(
 ) -> StreamResolverParam:
     if param.kind not in {Parameter.KEYWORD_ONLY, Parameter.POSITIONAL_OR_KEYWORD}:
         raise ValueError(
-            f"Stream resolver {resolver_fqn_for_errors} includes unsupported keyword or variadic arg '{param.name}'"
+            f"Stream resolver '{resolver_fqn_for_errors}' includes unsupported keyword or variadic arg '{param.name}'"
         )
 
     annotation = annotation_parser.parse_annotation(param.name)
     if isinstance(annotation, StateWrapper):
         if is_windowed_resolver:
-            raise ValueError("Windowed resolvers cannot have state")
+            raise ValueError(
+                f"Windowed stream resolvers cannot have state, but '{resolver_fqn_for_errors}' requires state."
+            )
         default_value = get_state_default_value(
             state_typ=annotation.typ,
             declared_default=param.default,
@@ -798,7 +913,10 @@ def _parse_stream_resolver_param(
     if is_windowed_resolver and get_origin(annotation) in (list, List):
         item_typ = get_args(annotation)[0]
         if _is_stream_resolver_body_type(item_typ):
-            return StreamResolverParamMessageWindow(name=param.name, item_typ=item_typ)
+            return StreamResolverParamMessageWindow(name=param.name, typ=annotation)
+
+    if is_windowed_resolver and isclass(annotation) and issubclass(annotation, (pyarrow.Table, polars.DataFrame)):
+        return StreamResolverParamMessageWindow(name=param.name, typ=annotation)
 
     raise ValueError(
         (
@@ -885,6 +1003,7 @@ def parse_and_register_stream_resolver(
     mode: Optional[str] = None,
     environment: Optional[Union[List[str], str]] = None,
     machine_type: Optional[MachineType] = None,
+    message: Optional[Type[Any]] = None,
 ) -> StreamResolver[P, T]:
     fqn = f"{fn.__module__}.{fn.__name__}"
     annotation_parser = ResolverAnnotationParser(fn, caller_globals, caller_locals)
@@ -928,6 +1047,7 @@ def parse_and_register_stream_resolver(
         machine_type=machine_type,
         signature=signature,
         output=output_features,
+        message=message,
     )
 
     StreamResolver.registry.append(resolver)
