@@ -15,6 +15,7 @@ import pytz
 from dateutil import parser
 from pendulum.tz.timezone import Timezone
 from polars.internals.construction import is_namedtuple
+from typing_extensions import get_args
 
 from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.streams import Windowed
@@ -41,6 +42,17 @@ except ImportError:
 
 _log = logging.getLogger(__name__)
 
+_NoneType = type(None)
+
+
+def _unwrap_optional_if_needed(typ: Type):
+    if get_origin(typ) == Union and len(get_args(typ)) == 2:
+        has_none = any(d == _NoneType for d in get_args(typ))
+        if has_none:
+            return next((m for m in get_args(typ) if m != _NoneType), None)
+
+    return typ
+
 
 class FeatureCodec:
     def __init__(self):
@@ -212,6 +224,8 @@ class FeatureCodec:
         return dtype
 
     def _get_pl_dtype_for_type(self, underlying: type | Windowed, fqn: str) -> Union[pl.DataType, Type[pl.DataType]]:
+        # Polars seems to allow optional for any dtype, so we ignore it when computing dtypes
+        underlying = _unwrap_optional_if_needed(underlying)
         if isinstance(underlying, Windowed):
             # If the parent window feature appears in a dataframe, then it's likely in a streaming resolver
             # Use the underlying type for that windowed feature
