@@ -1,12 +1,13 @@
 import argparse
 import os
 import pathlib
-import warnings
 import typing
+import warnings
 from typing import Dict, List, NamedTuple, Optional, Tuple, Type, Union, get_args, get_origin
 
 from chalk.features import Feature, Features, FeatureSetBase
 from chalk.importer import get_directory_root, import_all_python_files_from_dir
+from chalk.streams import Windowed
 
 try:
     from types import UnionType
@@ -14,9 +15,14 @@ except ImportError:
     UnionType = None
 
 
+class ParsedAnnotation(NamedTuple):
+    annotation: str
+    include_in_protocol_cls: bool
+
+
 class ParsedFeaturesClass(NamedTuple):
     module: str
-    annotations: Dict[str, str]
+    annotations: Dict[str, ParsedAnnotation]
 
 
 class StubGenerator:
@@ -25,7 +31,9 @@ class StubGenerator:
         self._module_to_stubs: List[Type[Features]] = []
         self._imports: Dict[Tuple[str, ...], str] = {}  # Mapping of full module name to the name it is imported as
 
-    def _add_import(self, item: Union[str, type, typing._SpecialForm], import_as: Optional[str] = None) -> str:
+    def _add_import(
+        self, item: Union[str, type, typing._SpecialForm, Windowed], import_as: Optional[str] = None
+    ) -> str:
         """Add an import for item.
 
         If ``item`` is already imported, then it will not be imported again.
@@ -39,7 +47,8 @@ class StubGenerator:
         """
         if isinstance(item, typing._SpecialForm):
             item = str(item)
-
+        if isinstance(item, Windowed):
+            item = type(item)
         if isinstance(item, type):
             item = f"{item.__module__}.{item.__name__}"
         if not isinstance(item, str):
@@ -68,16 +77,22 @@ class StubGenerator:
         item = f"__stubgen_{'_'.join(item_split)}"
         return item
 
-    def _register_imports_for_annotation(self, annotation: type) -> str:
+    def _register_imports_for_annotation(self, annotation: Union[type, Windowed]) -> str:
         if annotation == type(None):
             return "None"
         if annotation.__module__ == "builtins":
+            assert isinstance(annotation, type)
             return annotation.__name__
         origin, args = get_origin(annotation), get_args(annotation)
         if origin is None:
             assert args == ()
             # Likely a "simple" annotation, like an enum class
-            return self._add_import(annotation)
+            if isinstance(annotation, Windowed):
+                origin_name = self._add_import(annotation, import_as="Windowed")
+                arg_name = self._register_imports_for_annotation(annotation._kind)
+                return f"{origin_name}[{arg_name}]"
+            else:
+                return self._add_import(annotation)
         # Some sort of generic class, like a collection, or optional
         if origin == UnionType:
             # Convert new-style type unions to old-style typing.Unions
@@ -87,19 +102,22 @@ class StubGenerator:
         return f"{origin_name}[" + ", ".join(arg_names) + "]"
 
     def register_features_class(self, features_cls: Type[Features]):
-        annotations: Dict[str, str] = {}
+        annotations: Dict[str, ParsedAnnotation] = {}
         for f in features_cls.features:
             assert isinstance(f, Feature)
             assert f.typ is not None
-            if f.is_autogenerated:
-                continue
             if f.typ.is_dataframe:
                 self._add_import("chalk.features.DataFrame", "DataFrame")
-                annotations[f.attribute_name] = "DataFrame"
+                annotations[f.attribute_name] = ParsedAnnotation(
+                    "DataFrame",
+                    include_in_protocol_cls=True,
+                )
             else:
                 # Make a best-effort to figure out imports.
-                # Type
-                annotations[f.attribute_name] = self._register_imports_for_annotation(f.typ.parsed_annotation)
+                annotations[f.attribute_name] = ParsedAnnotation(
+                    self._register_imports_for_annotation(f.typ.parsed_annotation),
+                    include_in_protocol_cls=not f.is_windowed_pseudofeature and not f.is_autogenerated,
+                )
         if features_cls.__name__ in self._parsed_feature_classes:
             raise ValueError(
                 f"Unable to generate stubs due to multiple features sets called {features_cls.__name__}. Feature set names must be unique. Please rename one of these feature sets."
@@ -166,7 +184,7 @@ class {feature_cls_name}Metaclass(FeaturesMeta):"""
                 lines.append(
                     f"""\
     @property
-    def {attribute}(self) -> Type[{annotation}]: ...
+    def {attribute}(self) -> Type[{annotation.annotation}]: ...
 """
                 )
             if len(features_cls.annotations) == 0:
@@ -188,7 +206,7 @@ class {feature_cls_name}(Features, metaclass={feature_cls_name}Metaclass):"""
             for attribute, annotation in features_cls.annotations.items():
                 lines.append(
                     f"""\
-        {attribute}: {annotation} = ...,"""
+        {attribute}: {annotation.annotation} = ...,"""
                 )
             lines.append(
                 """\
@@ -197,7 +215,7 @@ class {feature_cls_name}(Features, metaclass={feature_cls_name}Metaclass):"""
             for attribute, annotation in features_cls.annotations.items():
                 lines.append(
                     f"""\
-        self.{attribute}: {annotation}"""
+        self.{attribute}: {annotation.annotation}"""
                 )
             if len(features_cls.annotations) == 0:
                 lines.append(
@@ -210,12 +228,16 @@ class {feature_cls_name}(Features, metaclass={feature_cls_name}Metaclass):"""
                 f"""\
 class {feature_cls_name}Protocol(Protocol):"""
             )
+            has_proto_annotation = False
             for attribute, annotation in features_cls.annotations.items():
+                if not annotation.include_in_protocol_cls:
+                    continue
+                has_proto_annotation = True
                 lines.append(
                     f"""\
-    {attribute}: {annotation}"""
+    {attribute}: {annotation.annotation}"""
                 )
-            if len(features_cls.annotations) == 0:
+            if not has_proto_annotation:
                 lines.append(
                     f"""\
     ..."""
