@@ -9,8 +9,14 @@ import polars as pl
 import polars.testing
 import pytest
 
-from chalk.features import DataFrame, Filter, after, feature_time, features, has_many, has_one, unwrap_feature
-from chalk.features.resolver import ResolverAnnotationParser
+try:
+    import zoneinfo
+except ImportError:
+    # Zoneinfo was introduced in python 3.9
+    from backports import zoneinfo
+
+from chalk.features import DataFrame, Filter, after, feature, feature_time, features, has_many, has_one, unwrap_feature
+from chalk.utils.annotation_parsing import ResolverAnnotationParser
 
 
 @dataclass(frozen=True)
@@ -40,6 +46,7 @@ class Taco:
     hat: str
     topping_id: str
     ts: datetime = feature_time()
+    unzoned_ts: datetime = feature(dtype=pl.Datetime("us", None))
     nicknames: DataFrame[Nickname] = has_many(lambda: Nickname.id == Taco.id)
     topping: "Topping" = has_one(lambda: Topping.id == Taco.topping_id)
     foo: Foo
@@ -103,6 +110,42 @@ class TestDF:
         assert isinstance(item, datetime)
         assert item == now_ts
 
+    def test_timezoned_datetime_str(self):
+        df = DataFrame(
+            {
+                Taco.ts: [
+                    # with a timezone, it should be converted
+                    "2022-04-21T19:36:02.767566-07:00",
+                    # without a timezone, it should be interpreted as the timezone of the column
+                    "2022-04-21T19:36:02.767566",
+                ],
+                Taco.id: [1, 1],
+            }
+        )
+        # The ts column is utc, so this should be converted
+        assert df[Taco.ts].to_polars().collect().rows() == [
+            (datetime(2022, 4, 22, 2, 36, 2, 767566, zoneinfo.ZoneInfo("UTC")),),
+            (datetime(2022, 4, 21, 19, 36, 2, 767566, zoneinfo.ZoneInfo("UTC")),),
+        ]
+
+    def test_unzoned_datetime_str(self):
+        df = DataFrame(
+            {
+                Taco.unzoned_ts: [
+                    # with a timezone, it should be dropped
+                    "2022-04-21T19:36:02.767566-07:00",
+                    # without a timezone should also work
+                    "2022-04-21T19:36:02.767566",
+                ],
+                Taco.id: [1, 1],
+            }
+        )
+        # The ts column is utc, so this should be converted
+        assert df[Taco.unzoned_ts].to_polars().collect().rows() == [
+            (datetime(2022, 4, 21, 19, 36, 2, 767566),),
+            (datetime(2022, 4, 21, 19, 36, 2, 767566),),
+        ]
+
     def test_in(self):
         chalk_df = DataFrame(
             {
